@@ -11,7 +11,7 @@ import itertools
 import collections
 from abc import ABCMeta, abstractmethod
 
-from .base import RedisCollection
+from .base import RedisCollection, same_types
 
 
 class SetOperation(object):
@@ -58,7 +58,7 @@ class SetOperation(object):
             return set(c._data(pipe=pipe))
         return set(c)
 
-    def _op(self, new_key, return_cls, others):
+    def _op(self, key, return_cls, others):
         """:func:`op` wrapper. Takes care of proper transaction
         handling and result instantiation.
         """
@@ -70,7 +70,7 @@ class SetOperation(object):
             left = self.s
             right = others
 
-        def trans(pipe, new_key):
+        def trans(pipe):
             # retrieve
             data = self._to_set(left, pipe=pipe)
             other_sets = [self._to_set(o, pipe=pipe) for o in right]
@@ -80,9 +80,9 @@ class SetOperation(object):
             pipe.multi()
 
             # store within the transaction
-            return self.s._create_new(elements, key=new_key, cls=return_cls,
+            return self.s._create_new(elements, key=key, cls=return_cls,
                                       pipe=pipe)
-        return self.s._transaction_with_new(trans, new_key=new_key)
+        return self.s._transaction(trans, key)
 
     @abstractmethod
     def op(self, s, other_sets):
@@ -97,7 +97,7 @@ class SetOperation(object):
         """
         pass
 
-    def _redisop(self, new_key, return_cls, other_keys):
+    def _redisop(self, key, return_cls, other_keys):
         """:func:`redisop` wrapper. Takes care of proper transaction
         handling and result instantiation.
         """
@@ -109,16 +109,15 @@ class SetOperation(object):
             left = self.s.key
             right = other_keys
 
-        def trans(pipe, new_key):
+        def trans(pipe):
             # operation
             elements = self.redisop(pipe, left, right)
             pipe.multi()
 
             # store within the transaction
-            return self.s._create_new(elements, key=new_key, cls=return_cls,
+            return self.s._create_new(elements, key=key, cls=return_cls,
                                       pipe=pipe)
-        return self.s._transaction_with_new(trans, new_key=new_key,
-                                            extra_keys=other_keys)
+        return self.s._transaction(trans, key, *other_keys)
 
     @abstractmethod
     def redisop(self, pipe, key, other_keys):
@@ -136,7 +135,7 @@ class SetOperation(object):
         """
         pass
 
-    def _redisopstore(self, new_key, return_cls, other_keys):
+    def _redisopstore(self, key, return_cls, other_keys):
         """:func:`redisopstore` wrapper. Takes care of proper transaction
         handling and result instantiation.
         """
@@ -148,14 +147,12 @@ class SetOperation(object):
             left = self.s.key
             right = other_keys
 
-        def trans(pipe, new_key):
+        def trans(pipe):
             # operation & possible store (in self.redisopstore)
-            new = self.s._create_new(key=new_key, cls=return_cls)
-            # ! new_key is unprefixed, but new.key is prefixed
+            new = self.s._create_new(key=key, cls=return_cls)
             self.redisopstore(pipe, new.key, left, right)
             return new
-        return self.s._transaction_with_new(trans, new_key=new_key,
-                                            extra_keys=other_keys)
+        return self.s._transaction(trans, key, *other_keys)
 
     @abstractmethod
     def redisopstore(self, pipe, new_key, key, other_keys):
@@ -164,10 +161,9 @@ class SetOperation(object):
 
         :param pipe: Redis transaction pipe.
         :type pipe: :class:`redis.client.StrictPipeline`
-        :param new_key: Prefixed Redis key of the new collection (destination).
+        :param new_key: Redis key of the new collection (destination).
         :type new_key: string
-        :param key: Prefixed Redis key from the original collection
-                    (first operand).
+        :param key: Redis key from the original collection (first operand).
         :type key: string
         :param other_keys: Redis keys of all the other collections
                            participating in this operation (other operands).
@@ -294,6 +290,8 @@ class Set(RedisCollection, collections.MutableSet):
     `set <http://redis.io/commands#set>`_ type.
     """
 
+    _same_types = (collections.Set,)
+
     def __init__(self, *args, **kwargs):
         """
         :param data: Initial data.
@@ -317,9 +315,6 @@ class Set(RedisCollection, collections.MutableSet):
                         Of course, you can construct your own pickling object
                         (it can be class, module, whatever). Default
                         serialization implementation uses :mod:`pickle`.
-        :param prefix: Key prefix to use when working with Redis. Defaults
-                       to empty string.
-        :type prefix: str
 
         .. note::
             :func:`uuid.uuid4` is used for default key generation.
@@ -445,6 +440,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetDifference(self, return_cls=kwargs.get('return_cls'))
         return op(*others)
 
+    @same_types
     def __sub__(self, other):
         """Return a new set with elements in the set that are
         not in the *other*.
@@ -459,13 +455,10 @@ class Set(RedisCollection, collections.MutableSet):
             is performed completely in Redis. Otherwise it's performed
             in Python and results are sent to Redis.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.difference(other)
 
+    @same_types
     def __rsub__(self, other):
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetDifference(self, flipped=True)
         return op(other)
 
@@ -496,6 +489,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetDifference(self, update=True)
         op(*others)
 
+    @same_types
     def __isub__(self, other):
         """Update the set, removing elements found in *other*.
 
@@ -509,8 +503,6 @@ class Set(RedisCollection, collections.MutableSet):
             is performed completely in Redis. Otherwise it's performed
             in Python and results are sent to Redis.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetDifference(self, update=True)
         return op(other)
 
@@ -530,6 +522,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetIntersection(self, return_cls=kwargs.get('return_cls'))
         return op(*others)
 
+    @same_types
     def __and__(self, other):
         """Return a new set with elements common to the set and the *other*.
 
@@ -541,13 +534,10 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__sub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.intersection(other)
 
+    @same_types
     def __rand__(self, other):
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetIntersection(self, flipped=True)
         return op(other)
 
@@ -563,6 +553,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetIntersection(self, update=True)
         op(*others)
 
+    @same_types
     def __iand__(self, other):
         """Update the set, keeping only elements found in it and the *other*.
 
@@ -574,8 +565,6 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__isub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetIntersection(self, update=True)
         return op(other)
 
@@ -595,6 +584,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetUnion(self, return_cls=kwargs.get('return_cls'))
         return op(*others)
 
+    @same_types
     def __or__(self, other):
         """Return a new set with elements from the set and the *other*.
 
@@ -606,13 +596,10 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__sub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.union(other)
 
+    @same_types
     def __ror__(self, other):
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.union(other, return_cls=other.__class__)
 
     def _update(self, data, others=None, pipe=None):
@@ -636,6 +623,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetUnion(self, update=True)
         op(*others)
 
+    @same_types
     def __ior__(self, other):
         """Update the set, adding elements from the *other*.
 
@@ -647,8 +635,6 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__isub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetUnion(self, update=True)
         return op(other)
 
@@ -669,6 +655,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetSymmetricDifference(self, return_cls=kwargs.get('return_cls'))
         return op(other)
 
+    @same_types
     def __xor__(self, other):
         """Update the set, keeping only elements found in either set, but not
         in both.
@@ -681,13 +668,10 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__sub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.symmetric_difference(other)
 
+    @same_types
     def __rxor__(self, other):
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         return self.symmetric_difference(other, return_cls=other.__class__)
 
     def symmetric_difference_update(self, other):
@@ -707,6 +691,7 @@ class Set(RedisCollection, collections.MutableSet):
         op = SetSymmetricDifference(self, update=True)
         op(other)
 
+    @same_types
     def __ixor__(self, other):
         """Update the set, keeping only elements found in either set, but not
         in both.
@@ -719,8 +704,6 @@ class Set(RedisCollection, collections.MutableSet):
         .. note::
             The same behavior as at :func:`__isub__` applies.
         """
-        if not isinstance(other, collections.Set):  # collections.Set is ABC
-            raise TypeError('Only sets are supported as operand types.')
         op = SetSymmetricDifference(self, update=True)
         return op(other)
 
